package org.graphavik.api.traversal;

import com.tinkerpop.blueprints.Edge;
import com.tinkerpop.blueprints.Graph;
import com.tinkerpop.blueprints.Query;
import com.tinkerpop.blueprints.Vertex;

/**
 * Interface need to be implemented to allow {@link org.graphavik.api.annotations.GraphTraversal} execution. It is legal
 * to annotate only {@link Iterable}.
 * <p/>
 * Receive graph and {@link Vertex} or {@link Edge} depending of class of annotated field ({@link
 * org.graphavik.api.annotations.NodeEntity} and {@link org.graphavik.api.annotations.RelationshipEntity} accordingly).
 * <p/>
 * Do some processing with vertex and returns object to set for class field. If object type is {@link Vertex} or {@link
 * Edge} it will be mapped to entity or relationship class.
 * <p/>
 * <b>Warning</b>: it is illegal to call Graphavik method from implementation of {@link Traversal}
 * <p/>
 * <b>Warning</b>: be accurate with it, incorrect return value may break something
 * <p/>
 * TODO(Michail Nikolaev): add about conversion services
 *
 * @param <T> type of entity traversal return. May be POJO class - in such case it will be set to annotated field. if
 *            type is equal to {@link Vertex} it will be converted to {@link org.graphavik.api.annotations.NodeEntity},
 *            if type is equal to {@link Edge} it will be converted to {@link org.graphavik.api.annotations.RelationshipEntity}
 * @author Michail Nikolaev
 * @see org.graphavik.api.annotations.GraphTraversal
 * @see Query
 * @see Vertex
 * @since <b>version 0.1</b> <i>created: 09.10.12</i>
 */
public interface Traversal<T> {
    /**
     * Called in case when filed of class annotated with {@link org.graphavik.api.annotations.NodeEntity} annotated with
     * {@link org.graphavik.api.annotations.GraphTraversal}.
     *
     * @param params params declared at {@link org.graphavik.api.annotations.GraphTraversal#params()}
     * @param vertex vertex represents target {@link org.graphavik.api.annotations.NodeEntity}  class
     * @param graph  graph to use
     * @return {@link Iterable} for objects to be used as value for annotated field, see class comments * @throws
     * @throws UnsupportedOperationException if traversal from vertex not supported
     */
    Iterable<T> processVertex(String[] params, Vertex vertex, Graph graph) throws UnsupportedOperationException;

    /**
     * Called in case when filed of class annotated with {@link org.graphavik.api.annotations.RelationshipEntity}
     * annotated with {@link org.graphavik.api.annotations.GraphTraversal}.
     *
     * @param params params declared at {@link org.graphavik.api.annotations.GraphTraversal#params()}
     * @param edge   edge represents target {@link org.graphavik.api.annotations.RelationshipEntity}  class
     * @param graph  graph to use
     * @return {@link Iterable} for objects to be used as value for annotated field, see class comments
     * @throws UnsupportedOperationException if traversal from edge not supported
     */
    Iterable<T> processEdge(String[] params, Edge edge, Graph graph) throws UnsupportedOperationException;

    /**
     * Specifies class of result.
     *
     * @return class of results of call to {@link Traversal#processEdge(String[], com.tinkerpop.blueprints.Edge,
     *         com.tinkerpop.blueprints.Graph)} and {@link Traversal#processVertex(String[],
     *         com.tinkerpop.blueprints.Vertex, com.tinkerpop.blueprints.Graph)}
     */
    Class<T> getTargetClass();

}
